# Copyright (c) 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Request Body limiting middleware.

"""

from oslo_config import cfg
import webob.dec
import webob.exc

from oslo_middleware._i18n import _
from oslo_middleware import base


_oldopts = [cfg.DeprecatedOpt('osapi_max_request_body_size',
                              group='DEFAULT'),
            cfg.DeprecatedOpt('max_request_body_size',
                              group='DEFAULT')]

_opts = [
    # default request size is 112k
    cfg.IntOpt('max_request_body_size',
               default=114688,
               help='The maximum body size for each '
                    ' request, in bytes.',
               deprecated_opts=_oldopts)
]


class LimitingReader(object):
    """Reader to limit the size of an incoming request."""
    def __init__(self, data, limit):
        """Initiates LimitingReader object.

        :param data: Underlying data object
        :param limit: maximum number of bytes the reader should allow
        """
        self.data = data
        self.limit = limit
        self.bytes_read = 0

    def __iter__(self):
        for chunk in self.data:
            self.bytes_read += len(chunk)
            if self.bytes_read > self.limit:
                msg = _("Request is too large.")
                raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)
            else:
                yield chunk

    def read(self, i=None):
        # NOTE(jamielennox): We can't simply provide the default to the read()
        # call as the expected default differs between mod_wsgi and eventlet
        if i is None:
            result = self.data.read()
        else:
            result = self.data.read(i)
        self.bytes_read += len(result)
        if self.bytes_read > self.limit:
            msg = _("Request is too large.")
            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)
        return result


class RequestBodySizeLimiter(base.ConfigurableMiddleware):
    """Limit the size of incoming requests."""

    def __init__(self, application, conf=None):
        super(RequestBodySizeLimiter, self).__init__(application, conf)
        self.oslo_conf.register_opts(_opts, group='oslo_middleware')

    @webob.dec.wsgify
    def __call__(self, req):
        max_size = self._conf_get('max_request_body_size')
        if (req.content_length is not None and
                req.content_length > max_size):
            msg = _("Request is too large.")
            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)
        if req.content_length is None:
            limiter = LimitingReader(req.body_file, max_size)
            req.body_file = limiter
        return self.application



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""oslo.i18n integration module.

See https://docs.openstack.org/oslo.i18n/latest/user/index.html

"""

import oslo_i18n


_translators = oslo_i18n.TranslatorFactory(domain='oslo_middleware')

# The primary translation function using the well-known name "_"
_ = _translators.primary



# Copyright 2011 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Base class(es) for WSGI Middleware."""

import six

if six.PY2:
    from inspect import getargspec as getfullargspec
else:
    from inspect import getfullargspec

import webob.dec
import webob.request
import webob.response

from oslo_config import cfg


class NoContentTypeResponse(webob.response.Response):

    default_content_type = None  # prevents webob assigning content type


class NoContentTypeRequest(webob.request.Request):

    ResponseClass = NoContentTypeResponse


class ConfigurableMiddleware(object):
    """Base WSGI middleware wrapper.

    These classes require an application to be initialized that will be called
    next.  By default the middleware will simply call its wrapped app, or you
    can override __call__ to customize its behavior.
    """

    @classmethod
    def factory(cls, global_conf, **local_conf):
        """Factory method for paste.deploy.

        :param global_conf: dict of options for all middlewares
                            (usually the [DEFAULT] section of the paste deploy
                            configuration file)
        :param local_conf: options dedicated to this middleware
                           (usually the option defined in the middleware
                           section of the paste deploy configuration file)
        """
        conf = global_conf.copy() if global_conf else {}
        conf.update(local_conf)

        def middleware_filter(app):
            return cls(app, conf)

        return middleware_filter

    def __init__(self, application, conf=None):
        """Base middleware constructor

        :param  conf: a dict of options or a cfg.ConfigOpts object
        """
        self.application = application

        # NOTE(sileht): If the configuration come from oslo.config
        # just use it.
        if isinstance(conf, cfg.ConfigOpts):
            self.conf = {}
            self.oslo_conf = conf
        else:
            self.conf = conf or {}
            if "oslo_config_project" in self.conf:
                if 'oslo_config_file' in self.conf:
                    default_config_files = [self.conf['oslo_config_file']]
                else:
                    default_config_files = None

                if 'oslo_config_program' in self.conf:
                    program = self.conf['oslo_config_program']
                else:
                    program = None

                self.oslo_conf = cfg.ConfigOpts()
                self.oslo_conf([],
                               project=self.conf['oslo_config_project'],
                               prog=program,
                               default_config_files=default_config_files,
                               validate_default_values=True)

            else:
                # Fallback to global object
                self.oslo_conf = cfg.CONF

    def _conf_get(self, key, group="oslo_middleware"):
        if key in self.conf:
            # Validate value type
            self.oslo_conf.set_override(key, self.conf[key], group=group)
        return getattr(getattr(self.oslo_conf, group), key)

    @staticmethod
    def process_request(req):
        """Called on each request.

        If this returns None, the next application down the stack will be
        executed. If it returns a response then that response will be returned
        and execution will stop here.
        """
        return None

    @staticmethod
    def process_response(response, request=None):
        """Do whatever you'd like to the response."""
        return response

    @webob.dec.wsgify(RequestClass=NoContentTypeRequest)
    def __call__(self, req):
        response = self.process_request(req)
        if response:
            return response
        response = req.get_response(self.application)

        args = getfullargspec(self.process_response)[0]
        if 'request' in args:
            return self.process_response(response, request=req)
        return self.process_response(response)


class Middleware(ConfigurableMiddleware):
    """Legacy base WSGI middleware wrapper.

    Legacy interface that doesn't pass configuration options
    to the middleware when it's loaded via paste.deploy.
    """

    @classmethod
    def factory(cls, global_conf, **local_conf):
        """Factory method for paste.deploy."""
        return cls



# Copyright (c) 2016 Cisco Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

import mock
from oslotest import base as test_base
import statsd
import webob.dec
import webob.exc

from oslo_middleware import stats


class TestStaticMethods(test_base.BaseTestCase):

    def test_removes_uuid(self):
        # Generate a long-format UUID (standard form).
        id = str(uuid.uuid4())
        path = "foo.{uuid}.bar".format(uuid=id)
        stat = stats.StatsMiddleware.strip_uuid(path)
        self.assertEqual("foo.bar", stat)

    def test_removes_short_uuid(self):
        id = uuid.uuid4().hex
        path = "foo.{uuid}.bar".format(uuid=id)
        stat = stats.StatsMiddleware.strip_short_uuid(path)
        self.assertEqual("foo.bar", stat)

    def test_strips_dots_from_version(self):
        path = "/v1.2/foo.bar/bar.foo"
        stat = stats.StatsMiddleware.strip_dot_from_version(path)
        self.assertEqual("/v12/foo.bar/bar.foo", stat)


class TestStatsMiddleware(test_base.BaseTestCase):

    def setUp(self):
        super(TestStatsMiddleware, self).setUp()
        self.patch(statsd, 'StatsClient', mock.MagicMock())

    def make_stats_middleware(self, stat_name=None, stats_host=None,
                              remove_uuid=False, remove_short_uuid=False):
        if stat_name is None:
            stat_name = uuid.uuid4().hex
        if stats_host is None:
            stats_host = uuid.uuid4().hex

        conf = dict(
            name=stat_name,
            stats_host=stats_host,
            remove_uuid=remove_uuid,
            remove_short_uuid=remove_short_uuid,
        )

        @webob.dec.wsgify
        def fake_application(req):
            return 'Hello, World'

        return stats.StatsMiddleware(fake_application, conf)

    def perform_request(self, app, path, method):
        req = webob.Request.blank(path, method=method)
        return req.get_response(app)

    def test_sends_counter_to_statsd(self):
        app = self.make_stats_middleware()
        path = '/test/foo/bar'

        self.perform_request(app, path, 'GET')

        expected_stat = "{name}.{method}.{path}".format(
            name=app.stat_name, method='GET',
            path=path.lstrip('/').replace('/', '.'))
        app.statsd.timer.assert_called_once_with(expected_stat)

    def test_strips_uuid_if_configured(self):
        app = self.make_stats_middleware(remove_uuid=True)
        random_uuid = str(uuid.uuid4())
        path = '/foo/{uuid}/bar'.format(uuid=random_uuid)

        self.perform_request(app, path, 'GET')

        expected_stat = "{name}.{method}.foo.bar".format(
            name=app.stat_name, method='GET')
        app.statsd.timer.assert_called_once_with(expected_stat)

    def test_strips_short_uuid_if_configured(self):
        app = self.make_stats_middleware(remove_short_uuid=True)
        random_uuid = uuid.uuid4().hex
        path = '/foo/{uuid}/bar'.format(uuid=random_uuid)

        self.perform_request(app, path, 'GET')

        expected_stat = "{name}.{method}.foo.bar".format(
            name=app.stat_name, method='GET')
        app.statsd.timer.assert_called_once_with(expected_stat)

    def test_strips_both_uuid_types_if_configured(self):
        app = self.make_stats_middleware(
            remove_uuid=True, remove_short_uuid=True)
        random_short_uuid = uuid.uuid4().hex
        random_uuid = str(uuid.uuid4())
        path = '/foo/{uuid}/bar/{short_uuid}'.format(
            uuid=random_uuid, short_uuid=random_short_uuid)

        self.perform_request(app, path, 'GET')

        expected_stat = "{name}.{method}.foo.bar".format(
            name=app.stat_name, method='GET')
        app.statsd.timer.assert_called_once_with(expected_stat)

    def test_always_mutates_version_id(self):
        app = self.make_stats_middleware()
        path = '/v2.1/foo/bar'

        self.perform_request(app, path, 'GET')

        expected_stat = "{name}.{method}.v21.foo.bar".format(
            name=app.stat_name, method='GET')
        app.statsd.timer.assert_called_once_with(expected_stat)

    def test_empty_path_has_sane_stat_name(self):
        app = self.make_stats_middleware()
        path = '/'

        self.perform_request(app, path, 'GET')

        expected_stat = "{name}.{method}".format(
            name=app.stat_name, method='GET')
        app.statsd.timer.assert_called_once_with(expected_stat)



# Copyright 2011 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os

from oslo_middleware.healthcheck import opts
from oslo_middleware.healthcheck import pluginbase

LOG = logging.getLogger(__name__)


class DisableByFilesPortsHealthcheck(pluginbase.HealthcheckBaseExtension):
    """DisableByFilesPorts healthcheck middleware plugin

    This plugin checks presence of a file that is provided for a application
    running on a certain port to report if the service is unavailable
    or not.

    Example of middleware configuration:

    .. code-block:: ini

      [filter:healthcheck]
      paste.filter_factory = oslo_middleware:Healthcheck.factory
      path = /healthcheck
      backends = disable_by_files_ports
      disable_by_file_paths = 5000:/var/run/keystone/healthcheck_disable, \
            35357:/var/run/keystone/admin_healthcheck_disable
      # set to True to enable detailed output, False is the default
      detailed = False
    """

    def __init__(self, *args, **kwargs):
        super(DisableByFilesPortsHealthcheck, self).__init__(*args, **kwargs)
        self.oslo_conf.register_opts(opts.DISABLE_BY_FILES_OPTS,
                                     group='healthcheck')
        self.status_files = {}
        paths = self._conf_get('disable_by_file_paths')
        self.status_files.update(self._iter_paths_ports(paths))

    @staticmethod
    def _iter_paths_ports(paths):
        for port_path in paths:
            port_path = port_path.strip()
            if port_path:
                # On windows, drive letters are followed by colons,
                # which makes split() return 3 elements in this case
                port, path = port_path.split(":", 1)
                port = int(port)
                yield (port, path)

    def healthcheck(self, server_port):
        path = self.status_files.get(server_port)
        if not path:
            LOG.warning('DisableByFilesPorts healthcheck middleware'
                        ' enabled without disable_by_file_paths set'
                        ' for port %s', server_port)
            return pluginbase.HealthcheckResult(available=True,
                                                reason="OK")
        else:
            if not os.path.exists(path):
                return pluginbase.HealthcheckResult(available=True,
                                                    reason="OK")
            else:
                return pluginbase.HealthcheckResult(available=False,
                                                    reason="DISABLED BY FILE")


class DisableByFileHealthcheck(pluginbase.HealthcheckBaseExtension):
    """DisableByFile healthcheck middleware plugin

    This plugin checks presence of a file to report if the service
    is unavailable or not.

    Example of middleware configuration:

    .. code-block:: ini

      [filter:healthcheck]
      paste.filter_factory = oslo_middleware:Healthcheck.factory
      path = /healthcheck
      backends = disable_by_file
      disable_by_file_path = /var/run/nova/healthcheck_disable
      # set to True to enable detailed output, False is the default
      detailed = False
    """

    def __init__(self, *args, **kwargs):
        super(DisableByFileHealthcheck, self).__init__(*args, **kwargs)
        self.oslo_conf.register_opts(opts.DISABLE_BY_FILE_OPTS,
                                     group='healthcheck')

    def healthcheck(self, server_port):
        path = self._conf_get('disable_by_file_path')
        if not path:
            LOG.warning('DisableByFile healthcheck middleware enabled '
                        'without disable_by_file_path set')
            return pluginbase.HealthcheckResult(
                available=True, reason="OK",
                details="No 'disable_by_file_path' configuration value"
                        " specified")
        elif not os.path.exists(path):
            return pluginbase.HealthcheckResult(
                available=True, reason="OK",
                details="Path '%s' was not found" % path)
        else:
            return pluginbase.HealthcheckResult(
                available=False, reason="DISABLED BY FILE",
                details="Path '%s' was found" % path)



# Copyright (c) 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
import mock
from oslotest import base as test_base

from oslo_middleware import correlation_id


class CorrelationIdTest(test_base.BaseTestCase):

    def setUp(self):
        super(CorrelationIdTest, self).setUp()

    def test_process_request(self):
        app = mock.Mock()
        req = mock.Mock()
        req.headers = {}

        mock_uuid4 = mock.Mock()
        mock_uuid4.return_value = "fake_uuid"
        self.useFixture(fixtures.MockPatch('uuid.uuid4', mock_uuid4))

        middleware = correlation_id.CorrelationId(app)
        middleware(req)

        self.assertEqual("fake_uuid", req.headers.get("X_CORRELATION_ID"))

    def test_process_request_should_not_regenerate_correlation_id(self):
        app = mock.Mock()
        req = mock.Mock()
        req.headers = {"X_CORRELATION_ID": "correlation_id"}

        middleware = correlation_id.CorrelationId(app)
        middleware(req)

        self.assertEqual("correlation_id", req.headers.get("X_CORRELATION_ID"))
